"use strict";
/* eslint no-console: off */
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const yargs = __importStar(require("yargs"));
const glob = __importStar(require("glob"));
const chalk_1 = __importDefault(require("chalk"));
const path_1 = require("path");
const _1 = require(".");
const argv = yargs
    .usage('Usage: $0 <fixtures> [options]')
    .option('cwd', {
    required: false,
    default: process.cwd(),
    normalize: true,
    type: 'string',
    describe: 'Working directory where the .graphqlconfig is located',
})
    .option('show-passes', {
    type: 'boolean',
    default: false,
    describe: 'Display passing fixtures along with failures in the console output',
})
    .help().argv;
_1.evaluateFixtures(glob.sync(argv._[0]), {
    cwd: argv.cwd,
})
    .then((evaluations) => {
    let passed = 0;
    let failed = 0;
    let lastFailed = false;
    console.log();
    evaluations.forEach((evaluation) => {
        const relativePath = path_1.relative(argv.cwd, evaluation.fixturePath);
        const formattedPath = `${chalk_1.default.dim(relativePath.replace(path_1.basename(relativePath), ''))}${chalk_1.default.bold(path_1.basename(relativePath))}`;
        if (evaluation.scriptError) {
            failed += 1;
            lastFailed = true;
            console.log(`${chalk_1.default.inverse.bold.red(' FAIL ')} ${formattedPath}`);
            console.log(evaluation.scriptError.message);
            if (evaluation.scriptError.stack) {
                console.log(chalk_1.default.dim(evaluation.scriptError.stack
                    .replace(evaluation.scriptError.message, '')
                    .replace(/^Error:\s*\n/, '')));
            }
            console.log();
        }
        else if (evaluation.validationErrors.length === 0) {
            passed += 1;
            lastFailed = false;
            if (argv.showPasses) {
                console.log(`${chalk_1.default.inverse.bold.green(' PASS ')} ${formattedPath}`);
            }
        }
        else {
            failed += 1;
            lastFailed = true;
            console.log(`${chalk_1.default.inverse.bold.red(' FAIL ')} ${formattedPath}`);
            evaluation.validationErrors.forEach((error) => {
                console.log(`${error.keyPath} ${chalk_1.default.red(error.message)}`);
            });
            console.log();
        }
    });
    if (passed + failed === 0) {
        console.log(chalk_1.default.bold.yellow('No tests run'));
        return;
    }
    if (!lastFailed) {
        console.log();
    }
    if (passed > 0) {
        console.log(chalk_1.default.bold.green(`${passed} passed`));
    }
    if (failed > 0) {
        console.log(chalk_1.default.bold.red(`${failed} failed`));
        process.exit(1);
    }
})
    .catch((error) => {
    console.log();
    console.log(`${chalk_1.default.inverse.bold.red(' ERROR ')} ${error.message}`);
    console.log(chalk_1.default.dim(error.stack.replace(error.message, '').replace(/^Error:\s*\n/, '')));
    process.exit(1);
});
