"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = require("fs-extra");
const path_1 = require("path");
const graphql_1 = require("graphql");
const graphql_config_1 = require("graphql-config");
const graphql_tool_utilities_1 = require("graphql-tool-utilities");
const validate_1 = require("./validate");
function evaluateFixtures(fixturePaths, { cwd }) {
    return __awaiter(this, void 0, void 0, function* () {
        const config = graphql_config_1.getGraphQLConfig(path_1.resolve(cwd));
        const projectASTCollection = yield Promise.all(graphql_tool_utilities_1.getGraphQLProjects(config).map(getOperationsForProject));
        return runForEachFixture(fixturePaths, (fixture) => evaluateFixture(fixture, projectASTCollection));
    });
}
exports.evaluateFixtures = evaluateFixtures;
function getOperationsForProject(projectConfig) {
    return __awaiter(this, void 0, void 0, function* () {
        const operationPaths = yield graphql_tool_utilities_1.getGraphQLProjectIncludedFilePaths(projectConfig);
        const operationSources = yield Promise.all(operationPaths.map(loadOperationSource));
        const document = graphql_1.concatAST(operationSources.map((source) => {
            try {
                return graphql_1.parse(source);
            }
            catch (error) {
                throw new Error(`Error parsing '${source.name}':\n\n${error.message.replace(/Syntax Error.*?\(.*?\) /, '')}`);
            }
        }));
        let schema;
        try {
            schema = projectConfig.getSchema();
        }
        catch (error) {
            throw new Error(`Error parsing '${projectConfig.schemaPath}':\n\n${error.message.replace(/Syntax Error.*?\(.*?\) /, '')}`);
        }
        return {
            ast: graphql_tool_utilities_1.compile(schema, document),
            config: projectConfig,
        };
    });
}
function loadOperationSource(filePath) {
    return __awaiter(this, void 0, void 0, function* () {
        const body = yield fs_extra_1.readFile(filePath, 'utf8');
        return new graphql_1.Source(body, filePath);
    });
}
function runForEachFixture(fixturePaths, runner) {
    return Promise.all(fixturePaths.map((fixturePath) => __awaiter(this, void 0, void 0, function* () {
        try {
            const fixture = yield fs_extra_1.readJSON(fixturePath);
            return Object.assign({ fixturePath }, runner({ path: fixturePath, content: fixture }));
        }
        catch (error) {
            return {
                fixturePath,
                scriptError: error,
                validationErrors: [],
            };
        }
    })));
}
function evaluateFixture(fixture, projectASTCollection) {
    try {
        const { operation, projectAST: { ast }, } = validate_1.getOperationForFixture(fixture, projectASTCollection);
        return validate_1.validateFixture(fixture, ast, operation);
    }
    catch (error) {
        if (error instanceof validate_1.MissingOperationError ||
            error instanceof validate_1.AmbiguousOperationNameError) {
            return {
                fixturePath: fixture.path,
                validationErrors: [error],
            };
        }
        throw error;
    }
}
